#include "p24fj48ga002.h"

//The Slave ADdress (SAD) associated to the LIS302DL is 0011100 (when SDO is tied to GND), 
//lsb indicates the R/W mode.
#define DEVICE_WRITE    0x38 //Default LIS302DL I2C address - write
#define DEVICE_READ     0x39 //Default LIS302DL I2C address - read

#define ACK     1
#define NO_ACK  0

/*************************************
**functions prototypes				**
*************************************/

void InitI2C(unsigned int FSCL)
{
	//To compute the Baud Rate Generator reload value, use the following equation:
	FCY = Fosc/2;
	I2C1BRG = (FCY/FSCL) - (FCY/10000000) - 1;
	
	
	//I2C module configuration
	I2C1CONbits.A10M = 0; //I2CxADD is a 7-bit slave address 
	I2C1CONbits.DISSLW = 0;//slew rate control of the device pin outpup
	I2C1CONbits.I2CEN = 1; //ENABLING I2C OPERATION
	
	
	writeRegister(0x20, 0b.0100.0111); //Power on device, enable all axis, and turn off self test - CTRL_REG1
}

void writeRegister(unsigned char regAdd, unsigned char regVal)
{ 
	while(I2C1STATbits.P == 0); //wait for previous stop bit 
	I2C1CONbits.SEN=1; 			//generate start bus event 
	while(I2C1CONbits.SEN==1); 	//wait for end of start 

	I2C1TRN= DEVICE_WRITE;				//send device address of slave in 
	while(I2C1STATbits.ACKSTAT==1); 	//wait for ACK from slave 
	while(I2C1STATbits.TRSTAT==1); 		//wait for end of ACK 

	I2C1TRN = regAdd; 					//Write register address
	while(I2C1STATbits.ACKSTAT==1); 	//wait for ACK from slave 
	while(I2C1STATbits.TRSTAT==1); 		//wait for end of ACK 

	I2C1TRN = regVal; 					//Write the data to register
	while(I2C1STATbits.ACKSTAT==1); 	//wait for ACK from slave 
	while(I2C1STATbits.TRSTAT==1); 		//wait for end of ACK 
} 

unsigned char readRegister(unsigned char regAdd)
{ 
	I2C1CONbits.RSEN=1;				//send restart 
	while(I2C1CONbits.RSEN==1); 	//wait for end of restart 
	
	I2C1TRN = DEVICE_WRITE; 		//send address to slave read mode 
	while(I2C1STATbits.ACKSTAT==1);	//wait for ACK from slave 
	while(I2C1STATbits.TRSTAT==1); 	//wait for end of ACK
	
	I2C1TRN = regAdd; 				//send register address
	while(I2C1STATbits.ACKSTAT==1);	//wait for ACK from slave 
	while(I2C1STATbits.TRSTAT==1); 	//wait for end of ACK
	
	I2C1TRN = DEVICE_READ; 		//send address to slave read mode 
	while(I2C1STATbits.ACKSTAT==1);	//wait for ACK from slave 
	while(I2C1STATbits.TRSTAT==1); 	//wait for end of ACK
	
	I2C1CONbits.ACKDT=1; 			//send NAK during acknowledge 
	I2C1CONbits.RCEN=1; 			//enable receive mode 
	while(I2C1STATbits.RBF==0); 	//wait for received datum 
	I2C1CONbits.ACKEN=1; 			//send acknowledge (NAK) 
	while(I2C1CONbits.ACKEN==1); 	//wait for end of acknowledge (NAK) 
	
	I2C1CONbits.PEN=1; 				//send stoP 
	while(I2C1CONbits.PEN==1);		//wait for end ofstoP 
	
	return(I2C1RCV); 
}
